/*
#pragma ident "@(#)Call.java	1.2      96/11/02     SMI"

 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package java.telephony;
import  java.telephony.capabilities.CallCapabilities;

/**
 * A Call represents a telephone call. A call can have zero or more
 * Connections: a two-party call has two connections, and a conference call
 * can have three or more connections. Each Connection represents an
 * association between the Call and an Address.
 * <p>
 * Applications create instances of a Call object with the
 * <EM>Provider.createCall()</EM> method. After this method returns, the
 * returned Call object has zero Connections associated with it and is in the
 * Call.IDLE state. The Call object maintains a reference to its Provider
 * object throughout the lifetime of the Call object. This Provider object does
 * not change throughout the lifetime of the Call object.
 * <p>
 * Call objects maintain an array of Connection objects which are associated
 * with the Call. A Call maintains a reference to a Connection object only
 * if it is not in the Connection.DISCONNECTED state. Therefore, if a Call has a
 * reference to a Connection, then that implies the Connection is not in
 * the Connection.DISCONNECTED state. When Connections in a Call move into the 
 * Connection.DISCONNECTED state (e.g. when a party hangs up), the Call loses
 * the reference to that Connection object.
 * <p>
 * A Call object in the Call.IDLE state has zero Connections associated with it.
 * All Call objects begin in the Call.IDLE state.
 * <p>
 * The Call object is in the Call.ACTIVE state when it has one or more
 * Connections associated with it. A Call may have Connections associated with
 * it through some external action or through an application successfully
 * invoking the <EM>Call.connect()</EM> method.
 * <p>
 * The Call object is in the Call.INVALID state when it has zero Connections
 * associated with it, but previously has Connections associated with it.
 * A Call in Call.INVALID state once had Connections which all have moved to
 * the Connection.DISCONNECTED state. A Call object in the Call.INVALID state
 * cannot be used further and should be deferenced by the application for
 * garbage collection. The Provider also loses its reference to the Call object
 * when the Call moves into the Call.INVALID state. This implies that the
 * Call object is no longer returns by <EM>Provider.getCalls()</EM>.
 * <p>
 * The predicates for the Call states are as follows. The purpose of the
 * predicates is to states truths about each Call state.
 * <p>
 * The IDLE state (this.getState() == Call.IDLE)
 * <OL>
 * <LI>this.getConnections() == null
 * </OL>
 * <p>
 * The ACTIVE state (this.getState() == Call.ACTIVE)
 * <OL>
 * <LI>Let Connection conn[] = this.getConnections()
 * <LI>conn != null && conn.length >= 1
 * <LI>For all i, conn[i].getState() != Connection.DISCONNECTED
 * </OL>
 * <p>
 * The INVALID state (this.getState() == Call.INVALID)
 * <OL>
 * <LI>this.getConnections() == null
 * </OL>
 * <p>
 * When the call is in the IDLE state, the call can be initialized by setting
 * optional parameters that influence the subsequent behavior of the call.
 * Various JTAPI extension packages provide methods to set such parameters.
 * <p>
 * The CallObserver interface reports all events pertaining to the Call object.
 * This not only includes state changes in the Call object, but also includes
 * state changes in all Connections associated with the Call object, and
 * state changes in all TerminalConnections associated with those Connections.
 * The CallObserver interface also reports when new Connections and
 * TerminalConnections are created in the Call. Applications instantiate
 * an object which implements the CallObserver interface and uses the
 * <EM>Call.addObserver()</EM> to begin the delivery of events to that
 * observer.
 * <p>
 * @see java.telephony.CallObserver 
 */

public interface Call {

  /**
   * The Call.IDLE state indicates the Call has zero Connections. It is the
   * initial state of all Call objects.
   */
  public static final int IDLE = 0x20;
  
  /**  
   * The Call.ACTIVE state indicates the Call has one or more Connections,
   * none of which are in the Connection.DISCONNECTED state.
   */
  public static final int ACTIVE = 0x21;
  
  /**  
   * The Call.INVALID state indicates the Call has lost all of its Connections.
   * A Call in this state cannot be used further.
   */
  public static final int INVALID = 0x22;


  /**
   * Returns the Connections associated with this call. Because of the
   * conditions mentioned above, none of the Connections returned will be in
   * the Connection.DISCONNECTED state. Also, if the Call is in the Call.IDLE
   * state or the Call.INVALID state, this method returns null. Otherwise, it
   * returns one or more Connections. The Provider must be in the
   * Provider.IN_SERVICE state for this method to be valid.
   * <p>
   * The pre-conditions for this method are listed below:
   * <OL>
   * <LI>(this.getProvider()).getState() == Provider.IN_SERVICE
   * </OL>
   * <p>
   * The post-conditions for this method are listed below:
   * <OL>
   * <LI>Let Connectionp[] conn = Call.getConnections()
   * <LI>if this.getState() == Call.IDLE then conn = null
   * <LI>if this.getState() == Call.INVALID then conn = null
   * <LI>if this.getState() == Call.ACTIVE then conn.length >= 1
   * <LI>For all i, conn[i].getState() != Connection.DISCONNECTED
   * </OL>
   * <p>
   * @return An array of the Connections associated with the call.
   * @exception InvalidStateException The Provider is not in the 
   * Provider.IN_SERVICE state.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public Connection[] getConnections()
    throws InvalidStateException, PlatformException;


  /**
   * Returns the provider associated with the call. The Provider object does
   * not change throughout the lifetime of the Call object. This method returns
   * the same value no matter what the Call state is. If the Call state is
   * Call.INVALID, this method returns the proper Provider object, however the
   * Provider object has lost its reference (see Provider.getCalls()) to this
   * Call object to faciliate the garbage collection of the Call object.
   * <p>
   * @return The Provider associated with the call.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public Provider getProvider()
    throws PlatformException;


  /**
   * Returns the current state of the telephone call. The state will be
   * one of the state constants defined above.
   * <p>
   * @return The current state of the call, either IDLE, ACTIVE, or INVALID.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public int getState()
    throws PlatformException;


  /**
   * Connect a pair of connections to a call. The Call must be in the Call.IDLE
   * state and its Provider must be in the Provider.IN_SERVICE state. Because
   * the Call must be in the Call.IDLE state for the success of this method,
   * the Call must by definition have no existing Connections associated with
   * it.
   * <p>
   * The first argument is the originating Terminal for the telephone call.
   * The second argument represents the Address from which the call is placed.
   * The originating address must be one of the addresses associated with the
   * originating Terminal, otherwise an exception is thrown. The third
   * argument is the string address to which the phone call is being placed.
   * This is also known as the destination address and must be a valid and
   * complete telephone address.
   * <p>
   * The pre-condition predicates for this method indicate which statements
   * must be true in order for this method to succeed. However, these
   * predicates do not guarantee the success of this method.
   * <p>
   * <OL>
   * <LI>(this.getProvider()).getState() == Provider.IN_SERVICE
   * <LI>this.getState() == Call.IDLE
   * </OL>
   * <p>
   * The post-condition predicates for the Call.connect() method are as
   * follows. Note that the eventual outcome of this method is described by
   * a number of scenarios below. These post-conditions only reflect the state
   * immediately after the <EM>Call.connect()</EM> returns.
   * <p>
   * <OL>
   * <LI>(this.getProvider()).getState() == Provider.IN_SERVICE
   * <LI>this.getState() == Call.ACTIVE
   * <LI>Let Connection c[] = this.getConnections()
   * <LI>c.length == 2
   * <LI>c[0].getState() == Connection.IDLE
   * <LI>c[1].getState() == Connection.IDLE
   * </OL>
   * <p>
   * Two scenarios of the eventual outcome of this method are given below.
   * Note, what is termed the "normal" operation of this method results in an
   * active telephone call. A sequence of discrete call model changes is given
   * by a list of discrete steps each labelled with a number. Changes in the
   * call model falling under the same number are seen by the application as
   * happening at once. These two scenarios do not represent all the possible
   * outcomes of the <EM>Call.connect</EM> method. These scenarios just
   * represent two typical scenarios.
   * <p>
   * Scenario 1: The "normal" operation of this method, where the result is
   * two ends participating in an active telephone call.
   * <p>
   * <OL>
   * <LI>The Call.connect() is method invoked with the given arguments.
   * <LI>Exactly two connections are created, each in the Connection.IDLE
   *    state. The first connection created represents the originating end of
   *   the call. The second connection created represents the destination end
   *   of the call.
   * <p>
   *    The Call.connect() method returns, returning a two-element array of
   *    Connections. The first element at index 0 represents the originating
   *    Connection. The second element at index 1 represents the destination
   *    Connection. Both Connections are in the Connection.IDLE state.
   * <p>
   *    The following events are delivered to the application via a
   *    CallObserver on this Call: a <EM>java.telephony.events.CallActiveEv</EM>, a
   *    <EM>java.telephony.events.ConnCreatedEv</EM> for each of the two Connections
   *    created.
   * <p>
   * <LI>Once the Provider begins to place the telephone call, the originating
   *    Connection moves from the Connection.IDLE state into the
   *    Connection.CONNECTED state. A TerminalConnection is created in the
   *    TerminalConnection.ACTIVE state between the originating Terminal and
   *    the Call.
   * <p>
   *    The following events are delivered to the application via a
   *    CallObserver on this Call: a <EM>java.telephony.events.ConnConnectedEv</EM>
   *    for the originating Connection, a
   *    <EM>java.telephony.events.TermConnCreatedEv</EM> for the new
   *    TerminalConnection created, and a
   *    <EM>java.telephony.events.TermConnActiveEv</EM> for the new Terminal
   *    Connection.
   * <p>
   *   Depending upon the configuration of the switch, additional
   *   TerminalConnections may be created. These TerminalConnections will be
   *   associated with Terminals which have the originating Address as one of
   *   their Addresseses. Unfortunately, there is no way for the implementation
   *   to tell which TerminalConnections will be created in advance, only that
   *   the number cannot be greater than the number of Terminals associated
   *   with the originating Address and each TerminalConnection must go to
   *   one of those Terminals associated with the originating Address. If such
   *   a Terminal Connection is created, it will be in the
   *   TerminalConnection.PASSIVE state.
   * <p>
   *   Additional events are delivered because of the above paragraph: a
   *   <EM>java.telephony.events.TermConnCreateEv</EM> for all additional
   *   Terminal Connections created, and a
   *   <EM>java.telephony.events.TermConnPassiveEv</EM> for each of those Terminal
   *   Connections.
   * <LI>The destination Connection moves from the Connection.IDLE state into
   *    the TerminalConnection.INPROGRESS state.
   * <p>
   *    The following events are delivered to the application via the
   *    CallObserver interface: a <EM>java.telephony.events.ConnInProgressEv</EM>
   *    for the destination Connection.
   * <LI>The destination Connection moves from the Connection.INPROGRESS state
   *    to the Connection.ALERTING state. TerminalConnection objects are
   *    created between the Call and every Terminal object associated with the
   *    destination Address in the TerminalConnection.RINGING state.
   * <p>
   *    The following events are delivered to the application via a CallObserver
   *    on this Call: a <EM>java.telephony.events.ConnAlertingEv</EM> for the
   *    destination Connection, a <EM>java.telephony.events.TermConnCreatedEv</EM>
   *    for all Terminal Connections created to Terminals with the destination
   *    Address, a <EM>java.telephony.events.TermConnRingingEv</EM> for all those
   *    new TerminalConnections.
   * <p>

   *    The following events are delivered to the application via the
   *    CallObserver on this Call: a <EM>java.telephony.events.ConnConnectedEv</EM>
   *    for the destination Connection, and a
   *    <EM>java.telephony.events.TermConnActiveEv</EM> for the TerminalConnection
   *    which answered the telephone call.
   * <p>
   *    For all other TerminalConnections in the TerminalConnection.RINGING
   *    state on the destination side, either one of two things will happen
   *    depending upon the configuration of the switch. Unfortunately, the
   *    implementation is unable to determine the behavior of the switch
   *    beforehand. In one situation, these TerminalConnections move from
   *    the TerminalConnection.RINGING state to the TerminalConnection.PASSIVE
   *    state. In the second situation, these TerminalConnections move from the
   *    TerminalConnection.RINGING state to the TerminalConnection.DROPPED
   *    state.
   * <p>
   *    Because of the paragraph above, the following events will be delivered
   *    to the application via the CallObserver interface on this Call:
   *    a <EM>java.telephony.events.TermConnPassiveEv</EM> in one situation, or
   *    a <EM>java.telephony.events.TermConnDroppedEv</EM> in the other situation.
   * </OL>
   * <p>
   * The post-condition predicates for scenario 1 are as follows. These
   * post-conditions state what is true about the call model after the final
   * step of scenario 1 completes.
   * <p>
   * <OL>
   * <LI>this.getState() == Call.ACTIVE
   * <LI>(this.getProvider()).getState() == Provider.IN_SERVICE
   * <LI>Let Connection c[] = this.getConnections()
   * <LI>c[0].getState == Connection.CONNECTED
   * <LI>c[1].getState == Connection.CONNECTED
   * <LI>Let TerminalConnection origtc[] = c[0].getTerminalConnections()
   * <LI>There exists an i, such that origtc[i].getState() ==
   * TerminalConnection.ACITVE.
   * <LI>Let TerminalConnection desttc[] = c[1].getTerminalConnections()
   * <LI>There exists an i, such that desttc[i].getState() ==
   * TerminalConnection.ACTIVE.
   * </OL>
   * <p>
   * Scenario 2: The desired telephone call is placed however the completion
   * of the call is impossible because of a failure. For example, this
   * scenario will occur if the destination address is busy. In this scenario
   * the destination Connection never reaches the ALERTING state. (i.e. the
   * destination party is never alerted to this telephone call)
   * <p>
   * <OL>
   * <LI>The Call.connect() is method invoked with the given arguments.
   * <LI>Exactly two connections are created, each in the Connection.IDLE
   *    state. The first connection created represents the originating end of
   *   the call. The second connection created represents the destination end
   *   of the call.
   * <p>
   *    The Call.connect() method returns, returning a two-element array of
   *    Connections. The first element at index 0 represents the originating
   *    Connection. The second element at index 1 represents the destination
   *    Connection. Both Connections are in the Connection.IDLE state.
   * <p>
   *    The following events are delivered to the application via a CallObserver
   *    on this Call: a <EM>java.telephony.events.CallActiveEv</EM>, a
   *    <EM>java.telephony.events.ConnCreatedEv</EM> for each of the two Connections
   *    created.
   * <p>
   * <LI>Once the Provider begins to place the telephone call, the originating
   *    Connection moves from the Connection.IDLE state into the
   *    Connection.CONNECTED state. A TerminalConnection is created in the
   *    TerminalConnection.ACTIVE state between the originating Terminal and
   *    the Call.
   * <p>
   *    The following events are delivered to the application via a CallObserver
   *    on this Call: a <EM>java.telephony.events.ConnConnectedEv</EM> for the
   *    originating Connection, a <EM>java.telephony.events.TermConnCreatedEv</EM>
   *    for the new TerminalConnection created, and a
   *    <EM>java.telephony.events.TermConnActiveEv</EM> for the new Terminal
   *    Connection.
   * <p>
   *   Depending upon the configuration of the switch, additional
   *   TerminalConnections may be created. These TerminalConnections will be
   *   associated with Terminals which have the originating Address as one of
   *   their Addresseses. Unfortunately, there is no way for the implementation
   *   to tell which TerminalConnections will be created in advance, only that
   *   the number cannot be greater than the number of Terminals associated
   *   with the originating Address and each TerminalConnection must go to
   *   one of those Terminals associated with the originating Address. If such
   *   a Terminal Connection is created, it will be in the
   *   TerminalConnection.PASSIVE state.
   * <p>
   *   Additional events are delivered because of the above paragraph: a
   *   <EM>java.telephony.events.TermConnCreateEv</EM> for all additional
   *   Terminal Connections created, and a
   *   <EM>java.telephony.events.TermConnPassiveEv</EM> for each of those Terminal
   *   Connections. 
   *
   * <LI>The destination Connection moves from the Connection.IDLE state into
   *    the TerminalConnection.INPROGRESS state.
   * <p>
   *    The following events are delivered to the application via the
   *    CallObserver interface: a <EM>java.telephony.events.ConnInProgressEv</EM>
   *    for the destination Connection.
   * <LI>The destination Connection moves from the Connection.INPROGRESS state
   *    into the Connection.FAILED state.
   * <p>
   *    The following events are delivered to the application via the
   *    CallObserver interface: a <EM>java.telephony.events.ConnFailedEv</EM> for
   *    the destination Connection.
   * </OL>
   * <p>
   * The post condition predicates for scenario 2 are as follows. These
   * post-conditions state what is true about the call model after the final
   * step of scenario 1 completes.
   * <OL>
   * <LI>this.getState() == Call.ACTIVE
   * <LI>(this.getProvider()).getState() == Provider.IN_SERVICE
   * <LI>Let Connection c[] = this.getConnections()
   * <LI>c[0].getState() == Connection.CONNECTED
   * <LI>c[1].getState() == Connection.FAILED
   * <LI>Let TerminalConnection origtc[] = c[0].getTerminalConnections()
   * <LI>There exists an i, such that origtc[i].getState() ==
   * TerminalConnection.ACITVE.
   * </OL>
   * <p>
   * @see java.telephony.events.CallActiveEv
   * @see java.telephony.events.ConnAlertingEv
   * @see java.telephony.events.ConnConnectedEv
   * @see java.telephony.events.ConnCreatedEv
   * @see java.telephony.events.ConnInProgressEv
   * @see java.telephony.events.TermConnActiveEv
   * @see java.telephony.events.TermConnCreatedEv
   * @see java.telephony.events.TermConnDroppedEv
   * @see java.telephony.events.TermConnPassiveEv
   * @see java.telephony.events.TermConnRingingEv
   * <p>
   * @param origterm The originating Terminal for this telephone call.
   * @param origaddr The originating Address for this telephone call.
   * @param destaddr The destination Address for this telephone call.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for placing the phone call is unavailable.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to place a telephone call.
   * @exception InvalidPartyException Either the originator or the destination
   * does not represent a valid party required to place a telephone call.
   * @exception InvalidArgumentException An argument provided is not valid
   * either by not providing enough information for connect() or is
   * inconsistent with another argument.
   * @exception InvalidStateException Some object required by this method is
   * not is a valid state as designated by the pre-conditions for this method.
   * @exception MethodNotSupportedException The implementation does not
   * support this method.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public Connection[] connect(Terminal origterm, Address origaddr,
			      String dialedDigits)
    throws ResourceUnavailableException, PrivilegeViolationException,
      InvalidPartyException, InvalidArgumentException, InvalidStateException,
      MethodNotSupportedException, PlatformException;


  /**
   * Adds an observer to the Call. The CallObserver reports all Call-related
   * state changes as events as well as all Connection state changes and
   * TerminalConnection state changes.  The Call object will report events to
   * this CallObserver object for the lifetime of the Call object or until the
   * observer is removed with the <EM>Call.removeObserver()</EM> method or
   * until the Call moves into the Call.INVALID state. The Call must be either
   * in the Call.IDLE state or Call.ACTIVE state as a pre-condition for this
   * method.
   * <p>
   * By default, when an observer is added to a telephone call, the first
   * batch of events may be a "snapshot". That is, if the observer was added
   * certain state changes in the call, the first batch of events will inform
   * the application of the current state of the call. Note that these
   * snapshot events do NOT provide a history of all events on the call. Rather
   * they provide the minimum neccessary event to bring the application
   * up-to-date with the current state of the call.
   * <p>
   * If an application attempts to add an instance of an observer already
   * present on this Call, then repeated attempts to add the instance
   * of the observer will silently fail, i.e. multiple instances of an
   * observer are not added and no exception will be thrown.
   * <p>
   * There are no pre-conditions for this method.
   * <p>
   * The post-condition predicates for this method are:
   * <OL>
   * <LI>observer is an element of this.getObservers()
   * </OL>
   * <p>
   * @param observer The observer being added.
   * @exception ResourceUnavailableException The resource limit for the
   * numbers of observers has been exceeded.
   * @exception PlatformException A platform-specific exception occurred.
   */ 
  public void addObserver(CallObserver observer)
    throws ResourceUnavailableException, PlatformException;


  /**
   * Returns a list of all CallObservers associated with this call object.
   * If no observers are on this Call object, then this method returns null.
   * <p>
   * There are no pre-conditions for this method.
   * <p>
   * The post-conditions for this method are:
   * <OL>
   * <LI>Let CallObserver[] obs = this.getObservers()
   * <LI>obs == null or obs.length >= 1
   * </OL>
   * <p>
   * @return An array of CallObserver objects associated with this
   * Call.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public CallObserver[] getObservers()
    throws PlatformException;


  /**
   * Removes the given observer from the Call. If successful, the observer
   * will no longer receive events generated by this call object.
   * <p>
   * Also, if an observer is not part of the Call, then this method
   * fails silently, i.e. no observer is removed an no exception is thrown.
   * <p>
   * There are no pre-conditions for this method.
   * <p>
   * The post-condition predicates for this method are:
   * <OL>
   * <LI>observer is not an element of this.getObservers()
   * </OL>
   * <p>
   * @param observer The observer which is being removed.
   * @exception PlatformException A platform-specific exception occurred.
   */ 
  public void removeObserver(CallObserver observer)
    throws PlatformException;


 /**
  * Gets the CallCapabilities object with respect to a Terminal and an
  * Address.
  * If null is passed as a Terminal parameter, the general/provider-wide Call 
  * capabilities are returned.
  * <p>
  */
  public CallCapabilities getCallCapabilities(Terminal terminal, Address address)
    throws InvalidArgumentException, PlatformException;
}

