/*
#pragma ident "@(#)Provider.java	1.34      97/02/06     SMI"

 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package java.telephony;
import  java.telephony.capabilities.*;
import  java.lang.*;

/**
 * A Provider represents the telephony-software-entity that interfaces with a 
 * telephony subsystem. The telephony subsystem could be a PBX connected to 
 * a server machine, or a telephony/fax card in a desktop machine or a 
 * networking technology such as IP.
 * <p>
 * A Provider instance is created and returned by the <EM>JtapiPeer</EM> 
 * method <EM>getProvider()</EM> which is given a string to describe the desired
 * Provider. This method sets up any needed communcation paths between the
 * application and the Provider object instance.
 * <p>
 * The Provider has methods for creating Call objects and obtaining Terminal
 * and Address objects. Applications cannot create their own instances of Call,
 * Terminal, Address, Connection, and TerminalConnection objects with the
 * "new" operator.
 * <p>
 * The Provider maintains lists of Call, Terminal, and Address objects it
 * knows about. Applications may create new Call objects using the
 * <EM>Provider.createCall()</EM> method. The Provider maintains a reference
 * to this new Call object until the Call object moves into the
 * Provider.INVALID state. Applications may obtain a list of all Call
 * objects known by the Provider (which are either in the Provider.IDLE or 
 * Provider.ACTIVE state) using the <EM>Provider.getCalls()</EM> method.
 * <p>
 * There may have been telephone calls present in the Provider's domain
 * before the Provider software object has been created by the application.
 * The Provider is still responsible, however, for reporting the existence
 * of these Calls when the application invokes the <EM>getCalls()</EM> method.
 * This principle applies to other objects as well. For example, the Address
 * object must report all Connections which are part of telephone calls even
 * if these Calls came into existence before the Provider.
 * <p>
 * Unlike Call objects, applications may not create Terminal or Address
 * objects. The Provider begins with knowledge of certain Terminal and Address
 * objects defined in the system. This list of Terminals and Addresses
 * represents those Terminals and Addresses within the Provider's domain.
 * This list is static once the Provider is created. The <EM>getTerminals()
 * </EM> and <EM>getAddresses()</EM> method returns these lists, respectively.
 * Other Addresses and Terminals may be created sometime during the operation
 * of the Provider when the Provider learns of new instances of these objects.
 * These new object, however, represent Addresses and Terminals outside the
 * Provider's domain. For example, if the Provider's domain is a PBX, the
 * Provider will know about all Addresses and Terminals in this PBX when the
 * Provider first starts. Any Addresses and Terminals it subsequently learns
 * about are outside this PBX. These Address and Terminal objects outside this
 * PBX are not reported via the <EM>getTerminals()</EM> and
 * <EM>getAddresses()</EM> methods.
 * <p>
 * The Provider may either be in the Provider.IN_SERVICE state, the
 * Provider.OUT_OF_SERVICE state, or the Provider.SHUTDOWN state. The Provider
 * state represents whether any action on that Provider or the objects
 * associated with the Provider are valid. The pre-condition that the Provider
 * state must be Provider.IN_SERVICE exists on most Java Telephony API object
 * methods and is noted in all instances. The Provider.OUT_OF_SERVICE state
 * indicates that the Provider is not available because of a temporary
 * condition not controlled by the application. The Provider.SHUTDOWN state
 * indicates that the Provider is permanently unavailable. The application may
 * use the <EM>Provider.shutdown()</EM> method to move the Provider into the
 * Provider.SHUTDOWN state itself or an external condition may cuase the
 * Provider to move into this state as well.
 * <p>
 * Each time a state changes occurs on a Provider object, the application is
 * notified via an <EM>event</EM>.  This event is reported via the
 * ProviderObserver interface. Applications instantiate objects which implement
 * the ProviderObject interface and use the <EM>Provider.addObserver()</EM>
 * method to begin the delivery of events. All Provider events reported via
 * this interface extend the ProviderEvent interface. Applications may then
 * query the event object returned for the specific state change. In the core
 * Provider API, the following generates events to the ProviderObserver:
 * when the Provider state changes to either Provider.IN_SERVICE,
 * Provider.OUT_OF_SERVICE, or Provider.SHUTDOWN.
 * <p>
 * It is not guaranteed or expected that objects (Call, Terminal, 
 * etc.) instantiated through one Provider will be usable with another
 * Provider. Thus an application that uses two providers must keep all the
 * objects relating to these providers separate.
 * <p>
 * @see java.telephony.JtapiPeer
 * @see java.telephony.ProviderObserver
 */

public interface Provider {

  /**
   * The Provider.IN_SERVICE state indicates that a Provider is currentl
   * available for use.
   */
  public static final int IN_SERVICE = 0x10;


  /** 
   * The Provider.OUT_OF_SERVICE state indicates that a Provider is temporarily
   * not available for use. Most methods in the Java Telephony API are invalid
   * when the Provider is in this state. Providers may come back IN_SERVICE
   * at any time, however, the application can take no direct action to cause
   * this change.
   */
  public static final int OUT_OF_SERVICE = 0x11;


  /**
   * The SHUTDOWN state indicates that a Provider is permanently no longer
   * available for use. Most methods in the Java TelephonyAPI are invalid when
   * the Provider is in this state.
   */
  public static final int SHUTDOWN = 0x12; 


  /**
   * Returns the current state of the Provider, either Provider.IN_SERVICE,
   * Provider.OUT_OF_SERVICE, or Provider.SHUTDOWN.
   * <p>
   * @return The current state of the provider.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public int getState()
    throws PlatformException;


  /**
   * Returns the unique string name of the Provider. Each different Provider must
   * have a unique string associated with it. This is the same string which the
   * application passed to the <EM>JtapiPeer.getProvider()</EM> method
   * to create this Provider instance.
   * <p>
   * @see java.telephony.JtapiPeer
   * <p>
   * @return The URL-string name of the Provider.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public String getName()
    throws PlatformException;
 

  /**
   * Returns an array of Call objects currently associated with the Provider.
   * When a Call moves into the Call.INVALID state, the Provider loses its
   * reference to this Call. Therefore, all Calls returned by this method
   * must either be in the Call.IDLE or Call.ACTIVE state. This method
   * returns null if the Provider has zero calls associated with it.
   * <p>
   * The pre-conditions for this method are:
   * <OL>
   * <LI>this.getState() == Provider.IN_SERVICE
   * </OL>
   * The post-conditions for this method are:
   * <OL>
   * <p>
   * <LI>this.getState() == Provider.IN_SERVICE
   * <LI>Let Calls calls[] = Provider.getCalls()
   * <LI>calls == null or calls.length >= 1
   * <LI>For all i, calls[i].getState() == Call.IDLE or Call.ACTIVE
   * </OL>
   * <p>
   * @return An array of Call objects currently known by this Provider.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public Call[] getCalls()
    throws PlatformException;


  /**
   * Returns an Address object which corresponds to the telephone number
   * string provided. If the provided name does not correspond to an
   * Address known by the Provider and within the Provider's domain,
   * the InvalidArgumentException is thrown. In other words, the Address
   * must appear in the list generated by Provider.getAddresses().
   * <p>
   * The pre-conditions for this method are:
   * <OL>
   * <LI>this.getState() == Provider.IN_SERVICE
   * <LI>Let Address addr = this.getAddress(number);
   * <LI>addr is an element of this.getAddresses();
   * </OL>
   * <p>
   * The post-conditions for this method are:
   * <OL>
   * <LI>this.getState() == Provider.IN_SERVICE
   * <LI>Let Address addr = this.getAddress(number);
   * <LI>addr is an element of this.getAddresses();
   * </OL>
   * <p>
   * @param number The telephone address string.
   * @return The Address object corresponding to the given telephone
   * number.
   * @exception InvalidArgumentException The name of the Address does not
   * correspond to the name of any Address object known to the Provider or
   * within the Provider's domain.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public Address getAddress(String number)
    throws InvalidArgumentException, PlatformException;


  /**
   * Returns a list of Addresses associated with the Provider and within the
   * Provider's domain. This list is static (i.e. is does not change) after
   * the Provider is first created. The Provider must be Provider.IN_SERVICE.
   * If no Address objects are associated with this Provider, then this method
   * returns null.
   * <p>
   * The pre-condition predicates for this method is:
   * <OL>
   * <LI>this.getState() == Provider.IN_SERVICE
   * </OL>
   * <p>
   * The post-condition predicates for this method is:
   * <OL>
   * <LI>this.getState() == Provider.IN_SERVICE
   * <LI>Let Address[] addr = this.getAddresses()
   * <LI>addr == null or addr.length >= 1
   * </OL>
   * <p>
   * @return An array of Addresses known by this provider.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public Address[] getAddresses()
    throws PlatformException;


  /**
   * Returns a list of Terminals associated with the Provider and within the
   * Provider's domain.  Each Terminal possesses a unique
   * name, which is assigned to it by the JTAPI implementation.
   * The Provider must be Provider.IN_SERVICE. If
   * there are no Terminals associated with this Provider, then this method
   * returns null.
   * <p>
   * The pre-condition predicate for this method is:
   * <OL>
   * <LI>this.getState() == Provider.IN_SERVICE
   * </OL>
   * <p>
   * The post-condition predicate for this method is:
   * <OL>
   * <LI>this.getState() == Provider.IN_SERVICE
   * <LI>Let Terminal[] term = this.getTerminals()
   * <LI>term == null or term.length >= 1
   * </OL>
   * <p>
   * @return An array of Terminals known by this provider.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public Terminal[] getTerminals()
    throws PlatformException;


  /**
   * Returns an instance of the Terminal class which corresponds to the
   * given name. Each Terminal has a unique name associated with it, which
   * is assigned to it by the JTAPI implementation.  If
   * no Terminal is available for the given name within the Provider's
   * domain, this method throws the InvalidArgumentException. This Terminal
   * must be in the array generated by <EM>Provider.getTerminals().</EM>
   * <p>
   * The pre-condition predicates for this method is:
   * <OL>
   * <LI>this.getState() == Provider.IN_SERVICE
   * <LI>this.getTerminal(name) is an element of this.getTerminals()
   * </OL>
   * <p>
   * The post-condition predicates for this method are:
   * <OL>
   * <LI>this.getState() == Provider.IN_SERVICE
   * <LI>this.getTerminal(name) is an element of this.getTerminals()
   * </OL>
   * <p>
   * @param name The name of desired Terminal object.
   * @return The Terminal object associated with the given name.
   * @exception InvalidArgumentException The name provided does not correspond
   * to a name of any Terminal known to the Provider or within the Provider's
   * domain.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public Terminal getTerminal(String name)
    throws InvalidArgumentException, PlatformException;


  /**
   * Instructs the Provider to shut itself down and perform all neccessary
   * cleanup. Applications invoke this method when they no longer intend to
   * use the Provider, most often right before they exit. This
   * method is intended to allow the Provider to perform any neccessary
   * cleanup which would not be taken care of when the Java objects are
   * garbage collected. This method causes the Provider to move into the
   * Provider.SHUTDOWN state, in which it will stay indefinitely.
   * <p>
   * The pre-conditions for this method are:
   * <OL>
   * <LI>this.getState() == Provider.IN_SERVICE
   * </OL>
   * <p>
   * The post-conditions for this method are:
   * <OL>
   * <LI>this.getState() == Provider.SHUTDOWN
   * </OL>
   * <p>
   * @exception InvalidStateException This Provider object is not in the
   * Provider.IN_SERVICE state.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public void shutdown()
    throws InvalidStateException, PlatformException;


  /**
   * Creates and returns a new instance of the Call object. This new call
   * object is in the Call.IDLE state and has no connections. An exception is
   * generated if a new call cannot be created.
   * <p>
   * The pre-condition predicates for this method are:
   * <OL>
   * <LI>this.getState() == Provider.IN_SERVICE
   * </OL>
   * <p>
   * The post-condition predicates for this method are:
   * <OL>
   * <LI>this.getState() == Provider.IN_SERVICE
   * <LI>Let Call call = this.createCall();
   * <LI>call.getState() == Call.IDLE.
   * <LI>call.getConnections() == null
   * <LI>call is an element of this.getCalls()
   * </OL>
   * <p>
   * @return The new Call object.
   * @exception ResourceUnavailableException An internal resource neccessary
   * to create a new Call object is unavailable.
   * @exception InvalidStateException The Provider is not in the
   * Provider.IN_SERVICE state.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to create a new telephone call object.
   * @exception MethodNotSupportedException The implementation does not support
   * creating new Call objects.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public Call createCall()
    throws ResourceUnavailableException, InvalidStateException,
      PrivilegeViolationException, MethodNotSupportedException,
      PlatformException;

  
  /**
   * Adds an observer to the Provider. The ProviderObserver reports all
   * Provider-related state changes as events. The Provider bject will report
   * events to this ProviderObserver object for the lifetime
   * of the Provider object or until the observer is removed with the
   * <EM>Provider.removeObserver()</EM> method.
   * <p>
   * This method is valid anytime and has no pre-conditions. Application must
   * have the ability to add observers to Providers so they can monitor
   * the changes in state in the Provider.
   * <p>
   * If an application attempts to add an instance of an observer already
   * present on this Provider, then repeated attempts to add the instance
   * of the observer will silently fail, i.e. multiple instances of an
   * observer are not added and no exception will be thrown.
   * <p>
   * The post-condition predicates for this method are:
   * <OL>
   * <LI>observer is an element of this.getObservers()
   * </OL>
   * <p>
   * @param observer The observer being added.
   * @exception ResourceUnavailableException The resource limit for the
   * numbers of observers has been exceeded.
   * @exception PlatformException A platform-specific exception occurred.
   */ 
  public void addObserver(ProviderObserver observer)
    throws ResourceUnavailableException, PlatformException;


  /**
   * Returns a list of all ProviderObservers associated with this call object.
   * If no observers exist on this Provider, then this method returns null.
   * <p>
   * There are no preconditions for this method. The application must be
   * able to get the list of observers even when the Provider is not in
   * the IN_SERVICE state.
   * <p>
   * The post-conditions for this method are:
   * <OL>
   * <LI>Let ProviderObserver[] obs = this.getObservers()
   * <LI>obs == null or obs.length >= 1
   * </OL>
   * <p>
   * @return An array of ProviderObserver objects associated with this
   * Provider.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public ProviderObserver[] getObservers()
    throws PlatformException;


  /**
   * Removes the given observer from the Provider. If successful, the observer
   * will no longer receive events generated by this call object.
   * <p>
   * There are no pre-conditions for this method. Application should be able
   * to remove observers even if the Provider is not in the IN_SERVICE state.
   * <p>
   * Also, if an observer is not part of the Provider, then this method
   * fails silently, i.e. no observer is removed an no exception is thrown.
   * <p>
   * The post-condition predicates for this method are:
   * <OL>
   * <LI>observer is not an element of this.getObservers()
   * </OL>
   * <p>
   * @param observer The observer which is being removed.
   * @exception PlatformException A platform-specific exception occurred.
   */ 
  public void removeObserver(ProviderObserver observer)
    throws InvalidArgumentException;


  /**
   * Returns the ProviderCapabilities object with respect to a Terminal.
   * If null is passed as a Terminal parameter, the general/provider-wide 
   * Provider capabilities
   * are returned.
   * <p>
   * @exception InvalidArgumentException invalid argument exception occurred
   * @exception PlatformException A platform-specific exception occurred.
   */
  public ProviderCapabilities getProviderCapabilities(Terminal term)
    throws InvalidArgumentException, PlatformException;


  /**
   * Gets the CallCapabilities object with respect to a Terminal.
   * If null is passed as a Terminal parameter, the general/provider-wide 
   * Call capabilities
   * are returned.
   * <p>
   * @param terminal the terminal whose call capabilities are being queried
   * @exception InvalidArgumentException invalid argument exception occurred
   * @exception PlatformException A platform-specific exception occurred.
   */
  public CallCapabilities getCallCapabilities(Terminal terminal,Address address)
    throws InvalidArgumentException, PlatformException;


  /**
   * Gets the ConnectionCapabilities object with respect to a Terminal.
   * If null is passed as a Terminal parameter, the general/provider-wide 
   * Connection capabilities
   * are returned.
   * <p>
   * @param terminal the terminal whose connection capabilities are being 
   * queried
   * @exception InvalidArgumentException invalid argument exception occurred
   * @exception PlatformException A platform-specific exception occurred.
   */
  public ConnectionCapabilities getConnectionCapabilities(Terminal terminal,Address address)
    throws InvalidArgumentException, PlatformException;


  /**
   * Gets the AddressCapabilities object with respect to a Terminal.
   * If null is passed as a Terminal parameter, the general/provider-wide 
   * Address capabilities
   * are returned.
   * <p>
   * @param terminal the terminal whose address capabilities are being 
   * queried
   * @exception InvalidArgumentException invalid argument exception occurred
   * @exception PlatformException A platform-specific exception occurred.
   */
  public AddressCapabilities getAddressCapabilities(Terminal terminal)
    throws InvalidArgumentException, PlatformException;


  /**
   * Gets the TerminalConnectionCapabilities of a Terminal.
   * If null is passed as a Terminal parameter, the general/provider-wide 
   * TerminalConnection capabilities
   * are returned.
   * <p>
   * @param terminal the terminal whose terminal connection capabilities 
   * are being queried
   * @exception InvalidArgumentException invalid argument exception occurred
   * @exception PlatformException A platform-specific exception occurred.
   */
  public TerminalConnectionCapabilities getTerminalConnectionCapabilities(Terminal terminal)
    throws InvalidArgumentException, PlatformException;


  /**
   * Gets the TerminalCapabilities object with respect to a Terminal.
   * If null is passed as a Terminal parameter, the general/provider-wide 
   * Terminal capabilities
   * are returned.
   * <p>
   * @param terminal the terminal whose capabilities are being queried
   * @exception InvalidArgumentException invalid argument exception occurred
   * @exception PlatformException A platform-specific exception occurred.
   */
  public TerminalCapabilities getTerminalCapabilities(Terminal terminal)
    throws InvalidArgumentException, PlatformException;
}
