/*
#pragma ident "@(#)CallControlCall.java	1.29      97/02/02     SMI"

 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package java.telephony.callcontrol;
import  java.telephony.*;

/**
 * The CallControlCall interface extends the core Call interface. The
 * CallControlCall interface provides more functionality on the Call object
 * for the purposes of the CallControl package.
 * <p>
 * Particularly, it provides the addParty() method call to conference
 * in an additional party, the drop() method to drop the entire Call, the
 * conference() method to conference two calls together, the transfer()
 * method to transfer a telephone call, and the consult() method to begin
 * a consultation telephone Call.
 */

public interface CallControlCall extends Call {

  /**
   * Returns the calling Address associated with this call. The calling
   * Address is defined as the Address which placed the telephone call.
   * <p>
   * If the calling address is unknown or not yet known, this method returns
   * null.
   * <p>
   * @return The calling Address.
   * @exception PlatformException Some platform-specific exception occured.
   */
  public CallControlAddress getCallingAddress()
    throws PlatformException;


  /**
   * Returns the calling Terminal associated with this Call. The calling
   * Terminal is defined as the Terminal which placed the telephone call.
   * <p>
   * If the calling Terminal is unknown or not yet know, this method returns
   * null.
   * <p>
   * @return The calling Terminal.
   * @exception PlatformException Some platform-specific exception occured.
   */
  public CallControlTerminal getCallingTerminal()
    throws PlatformException;


  /**
   * Returns the called Address associated with this Call. The called
   * Address is defined as the Address to which the call has been originally
   * placed.
   * <p>
   * If the called address is unknown or not yet known, this method returns
   * null.
   * <p>
   * @returns The called Address.
   * @exception PlatformException Some platform-specific exception occured.
   */
  public CallControlAddress getCalledAddress()
    throws PlatformException;


  /**
   * Returns the last redirected Address associated with this Call.
   * The last redirected Address is the Address at which the current telephone
   * call was placed immediately before the current Address. This is common
   * if a Call is forwarded to several Addresses before being answered.
   * <p>
   * If the the last redirected address is unknown or not yet known, this
   * method returns null.
   * <p>
   * @returns The last redirected Address for this telephone Call.
   * @exception PlatformException Some platform-specific exception occured.
   */
  public CallControlAddress getLastRedirectedAddress()
    throws PlatformException;


  /**
   * Adds an additional party to an existing telephone Call. This is sometimes
   * called a "single-step conference" because a party is conferenced into a
   * telephone call directly. The telephony address provided must be
   * complete and valid.
   * <p>
   * The Call must be ACTIVE before this method is valid. This Call must also
   * have at least two Connections in the CONNECTED state. An addition
   * restriction requires that at most one other Connection may be in either
   * the INPROGRESS state or ALERTING state. The Provider must also be
   * IN_SERVICE. The pre-conditions for this method are given by the following:
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>c = call.getConnections() && c.length >= 2
   * <LI>c = call.getConnections() && two connections.getState() == CONNECTED
   * <LI>there exists at most one c = call.getConnections() such that 
   * c.getState() equals ALERTING or INPROGRESS.
   * </OL>
   * <p>
   * This method creates a new Connection and associates it with the Call and
   * the Address object associated with the new party. This Connection is
   * in the IDLE state and is the return value for this method. The
   * post-conditions which are true when this method returns are as follows,
   * where 'c' is the returned connection.
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>c.getState() == IDLE
   * </OL>
   * <p>
   * In a typical scenario, this new Connection has the same sequence of state
   * transitions as a destination Connection when the method Call.connect()
   * is invoked. Documentation of that method provides detail of the possible
   * state transition scenarios of the new Connection created by this method.
   * <p>
   * On some platforms, switch limitations impose restrictions on the number of
   * connections of a particular state in a call. For instance, it is common to
   * restrict the number of ALERTING connections to at most one.  If any
   * operation is attempted that would violate one of these restrictions.
   * As a result, one precondition for this method is that there cannot be
   * another connection in either the INPROGRESS or ALERTING state. Although
   * some systems may not enforce this requirement, for consistency the Java
   * Telephony API chose this conservative requirement.
   * <p>
   * @param newParty The telephone number address of the party to be added.
   * @return The new Connection associated with the added party.
   * @exception InvalidStateException The state of some object is not valid
   * as designated by the pre-conditions for this method.
   * @exception InvalidPartyException The new party to be added to the call
   * is invalid.
   * @exception MethodNotSupportedException This method is not supported by
   * the implementation.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to invoke this method.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for the successful invocation of this method is not available.
   * @exception PlatformException A platform-specific exception occurred.
   */ 
  public CallControlConnection addParty(String newParty)
    throws InvalidStateException, InvalidPartyException,
           MethodNotSupportedException, PrivilegeViolationException,
           ResourceUnavailableException, PlatformException;
    
  
  /**
   * Drops the entire Call. This method is equivalent to using the
   * disconnect() method on each Connections which is part of the Call. There
   * are some Connections which the application does not possess the proper
   * authority to drop. In this case, the drop() method performs no action
   * on these. As a result of dropping some of the Connection, the other
   * Connections which the application cannot control may drop automatically
   * as a result. If all Connections succeed in moving to the DISCONNECTED
   * state, the call becomes INVALID. This method returns when it can
   * successfully drop as many Connections are possible. All of the
   * TerminalConnections associated with the dropped Connections will move
   * into the DROPPED state. 
   * <p>
   * The pre-conditions for this method are:
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * </OL>
   * <p>
   * The post-conditions for this method are:
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>if all connections = call.getConnections() and
   * connections.getState() == DISCONNECTED, then call.getState() == INVALID
   * <LI>for all connections = call.getConnections and connections.getState()
   * == DISCONNECTED, for all termconns = connections.getTerminalConnections(),
   * termconns.getState() == DROPPED.
   * </OL>
   * <p>
   * @exception InvalidStateException The state of some object is not valid
   * as designated by the pre-conditions for this method.
   * @exception MethodNotSupportedException This method is not supported by
   * the implementation.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to invoke this method and it can drop none of the
   * Connections.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for the successful invocation of this method is not available.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public void drop()
      throws InvalidStateException, MethodNotSupportedException,
	PrivilegeViolationException, ResourceUnavailableException,
	PlatformException;


  /**
   * Takes the originating end of a telephone call off-hook. This method
   * permits application to simply take the originating terminal of a telephone
   * call off-hook, so that users may manually dial telephone number digits
   * or applications may supply digits with the
   * <EM>CallControlConnection.addToAddress()</EM> method. This is in contrast
   * to the <EM>Call.connect()</EM> method which requires the complete
   * destination address string.
   * <p>
   * This Call object must be in the IDLE state prior to the invocation of
   * this method. This method creates a Connection to the originating Address
   * in the CallControlConnection.INITIATED state and returns this Connection
   * at the successful completion of this method. This method also creates
   * a TerminalConnection associated with the originating Terminal in the
   * CallControlTerminalConnection.TALKING state.
   * <p>
   * The pre-conditions for this method are:
   * <OL>
   * <LI>(this.getProvider()).getState() == Provider.IN_SERVICE
   * <LI>this.getState() == Call.IDLE
   * </OL>
   * <p>
   * The post-conditions for this method are:
   * <OL>
   * <LI>(this.getProvider()).getState() == Provider.IN_SERVICE
   * <LI>Let Connnection c = this.offHook(Address, Terminal)
   * <LI>c.getCallControlState() == CallControlConnection.INITIATED
   * <LI>Let TerminalConnection tc[] = c.getTerminalConnections()
   * <LI>tc.length == 1
   * <LI>tc[0].getCallControlState() == CallControlTerminalConnection.TALKING;
   * <LI>tc element of origterminal.getTerminalConnections()
   * <LI>c element of origaddress.getConnections()
   * <LI>c element of this.getConnections()
   * </OL>
   * <p>
   * @param origaddress The originating Address object.
   * @param origterminal The originating Terminal object.
   * @return The Connection associated with the originating end of the
   * telephone call.
   * @exception InvalidStateException The state of some object is not valid
   * as designated by the pre-conditions for this method.
   * @exception MethodNotSupportedException This method is not supported by
   * the implementation.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to invoke this method and it can drop none of the
   * Connections.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for the successful invocation of this method is not available.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public CallControlConnection offHook(Address origaddress, Terminal origterminal)
    throws InvalidStateException, MethodNotSupportedException,
      PrivilegeViolationException, ResourceUnavailableException,
      PlatformException;


  /**
   * Merges two Calls together resulting in the union of the parties involved
   * with both calls belong to a single telephone call. The telephone Call
   * provided as the argument has its Connections move to this Call object,
   * and the Call object argument becomes invalid.
   * <p>
   * In order for a conference action to be successful, there must exist
   * a Terminal object which is part of both telephone Calls. Furthermore,
   * this Terminal object will have two TerminalConnections associated with
   * it which are each part of one of the Calls. An application may have
   * specified on such TerminalConnections on this call to be the "conference
   * controller". If the application did not make such a selection, the
   * implementation chooses a suitable Terminal and TerminalConnection. We
   * shall call the TerminalConnection on this Call acting as the conference
   * controller (whether selected by the application or not) as "conftermconn"
   * and its Terminal as "confterm". The TerminalConnection associated with
   * confterm and the second Call object, we shall call "scndtermconn". The
   * following are the initial pre-condition predicates for this method:
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>conftermconn element of call.getConnections().getTerminalConnections()
   * <LI>conftermconn element of confterm.getTerminalConnections()
   * <LI>scndtermconn element of
   * othercall.getConnections().getTerminalConnections()
   * <LI>scndtermconn element of confterm.getTerminalConnections()
   * </OL>
   * <p>
   * These two TerminalConnections must be in a particular state. Specifically,
   * conftermconn must be in the CallControlTerminalConnection.ACTIVE state
   * and scndtermconn must be in the CallControlTerminalConnection.HELD state.
   * These additional pre-conditions are listed below:
   * <p>
   * <OL>
   * <LI>conftermconn.getState() == ACTIVE
   * <LI>scndtermcomm.getState() == HELD
   * </OL>
   * <p>
   * The conference() method "merges" these two telephone Calls together by
   * taking all of the Connections and TerminalConnections on otherCall and
   * creating new Connections and TerminalConnections with the same Address
   * and Terminal objects, respectively, in the same state as in oldCall. The
   * conftermconn and scndtermconn are the exception. The scndtermconn
   * TerminalConnection in otherCall moves to the DROPPED state. Let c equal
   * some Connections in otherCall and tc equal some TerminalConnection in
   * otherCall. Then, for all Connection's c and TerminalConnections tc
   * (with the exception tc = scndtermconn), the following post-conditions
   * indicate the result of the conference method. The construct new(c) and
   * new(tc) mean to create a new Connection and TerminalConnection object
   * with the same Address and TerminalConnection references.
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>new(c) element of call.getConnections()
   * <LI>new(c).getState() == c.getState()
   * <LI>new(tc) element of (call.getConnections()).getTerminalConnections()
   * <LI>new(tc).getState() == tc.getState()
   * </OL>
   * <p>
   * Finally, the Connections and TerminalConnections in otherCall move to
   * DISCONNECTED and DROPPED respectively, and otherCall becomes INVALID.
   * <p>
   * <OL>
   * <LI>c.getState() == DISCONNECTED
   * <LI>tc.getState() == DROPPED
   * <LI>otherCall.getState() == INVALID
   * </OL>
   * <p>
   * @param otherCall The second Call which to merge with this Call object.
   * @exception InvalidArgumentException The Call object provided is
   * not valid for the conference
   * @exception InvalidStateException The state of some object is not valid
   * as designated by the pre-conditions for this method.
   * @exception MethodNotSupportedException This method is not supported by
   * the implementation.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to invoke this method.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for the successful invocation of this method is not available.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public void conference(Call otherCall)
    throws InvalidStateException, InvalidArgumentException,
           MethodNotSupportedException, PrivilegeViolationException,
           ResourceUnavailableException, PlatformException;

  
  /**
   * This flavor of the transfer() method transfer all parties currently
   * associated with the Call to another telephone Call. Here, all parties
   * associated with the Call provided as the argument are moved to this
   * Call object.
   * <p>
   * There must be a Terminal which is part of both calls on the same Address.
   * This Terminal has a TerminalConnection to each of the telephone calls. If
   * more than one such Terminal exists, the implementation choose are suitable
   * one. If no such Terminal exists, this method throws an exception. Let
   * us call the TerminalConnection associated with this call 'termconn' and
   * the TerminalConnection associated with otherCall, 'scndtermconn'. 
   * <p>
   * These two TerminalConnections (and the Terminal) drop off of the Call
   * once the transfer takes place. The fact that this Terminal drops from the
   * call is a distinguishing feature between this transfer() method and
   * conference(). The TerminalConnections must either be in the TALKING or
   * HELD state (these are CallControlTerminalConnection states). The Provider
   * must be IN_SERVICE. These pre-conditions are listed below:
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>otherCall.getState() == ACTIVE
   * <LI>termconn element of (call.getConnections()).getTerminalConnections()
   * <LI>termconn.getState() == TALKING or HELD
   * <LI>scndtermconn element of
   * (otherCall.getConnections()).getTerminalConnections()
   * <LI>scndtermconn.getState() == TALKING or HELD
   * </OL>
   * <p>
   * This method moves the termconn and scndtermconn into the DROPPED state,
   * and if it is the only TerminalConnection on the Connection, the
   * Connection moves to the DISCONNECTED state. The sncdtermconn object
   * also moves into the TerminalConnection state and does not get moved to
   * this Call object. All other Connections and TerminalConnections which are
   * part of otherCall are moved to this Call object in the same state as
   * they were in otherCall. New Connection objects are created, and the old
   * Connection and TerminalConnection objects on otherCall move to
   * DISCONNECTED and DROPPED, respectively. The otherCall object becomes
   * INVALID. The post-conditions of this method are as follows:
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>termconn.getState() == DROPPED
   * <LI>(termconn.getConnection()).getTerminalConnections is of length 1, then
   * (termconn.getConnection()).getState() == DISCONNECTED
   * <LI>scndtermconn.getState() == DROPPED
   * <LI>For all connections in otherCall.getConnections(),call.getConnections()
   * has an element in the same state and the same Address reference.
   * <LI>For all terminalconnection in 
   * (otherCall.getConnections()).getTerminalConnections(), this call object
   * has the same number of TerminalConnections minus scndtermconn in the
   * same state and associated with the same Connection and Terminal.
   * <LI>For all connections in otherCall.getConnections(),
   * connections.getState() == DISCONNECTED
   * <LI>For all terminalconnections in connections.getTerminalConnections(),
   * terminalconnections.getState() == DROPPED
   * <LI>otherCall.getState() == INVALID
   * </OL>
   * <p>
   * @param otherCall The other Call which to transfer to this Call.
   * @exception InvalidArgumentException The TerminalConnection
   * controlling the transfer is not valid or does not exists or the other
   * telephone call is not valid.
   * @exception InvalidStateException The state of some object is not valid
   * as designated by the pre-conditions for this method.
   * @exception MethodNotSupportedException This method is not supported by
   * the implementation.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to invoke this method.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for the successful invocation of this method is not available.
   * @exception PlatformException A platform-specific exception occurred.
   * @exception Exception An implementation-specific exception occurred.
   * <p>
   * @exception PlatformException A platform-specific exception occurred.
   */
  public void transfer(Call otherCall)
    throws InvalidStateException, InvalidArgumentException,
      InvalidPartyException, MethodNotSupportedException,
      PrivilegeViolationException, ResourceUnavailableException,
      PlatformException;


  /**
   * This flavor of the transfer() method transfer all parties currently
   * associated with the Call to another telephony number address. Typically,
   * the participants of call will hear the new address alerting. The telephone
   * address provided must be valid and complete.
   * <p>
   * The transfer() method is associated with some TerminalConnection which
   * is part of the current telephony Call. This TerminalConnection specifies
   * from which Terminal is transfer is being invoked. This Terminal and
   * TerminalConnection drop off from the call once transferred. This special
   * TerminalConnection is called the "transfer controller" and is optionally
   * set by the setTransferController() method. If applications do not select
   * a transfer controller, then the implementations will select a suitable
   * one. For the purposes of this method, this transfer controller
   * TerminalConnection will be denoted as 'termconn'.
   * <p>
   * The transfer controller must be in the ACTIVE state and be part of this
   * telephone call. The Provider must also be IN_SERVICE. The pre-conditions
   * for this method are:
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>termconn element of (call.getConnections()).getTerminalConnections()
   * <LI>termconn.getState() == ACTIVE
   * </OL>
   * <p>
   * This method moves the termconn into the DROPPED state, and if it is
   * the only TerminalConnection on the Connection, the Connection moves to
   * the DISCONNECTED state. The transfer() method then creates a new
   * Connection in the IDLE state to an Address object corresponding to
   * the string telephone number address provided. The transfer() method
   * returns this new Connection. The post-conditions of this method are
   * as follows, given the returned Connection is denoted by 'connection':
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>termconn.getState() == DROPPED
   * <LI>(termconn.getConnection()).getTerminalConnections is of length 1, then
   * (termconn.getConnection()).getState() == DISCONNECTED
   * <LI>if connection != null, connection is an element of call.getConnections()
   * <LI>if connection != null, then connection.getState() == IDLE
   * </OL>
   * <p>
   * An application can expect the new connection to transition through the
   * Connection states similiar to a destination Connection does after a
   * Call.connect() is invoked.
   * <p>
   * Note that the new Connection created and returned may be null. This
   * Connection will be null in the case that the Call has transferred out of
   * the Provider's domain.
   * <p>
   * @param address The destination address where the Call is being
   * transferred.
   * @return The new Connection associated with the destination or null.
   * @exception InvalidArgumentException The TerminalConnection provided as
   * controlling the transfer is not valid or part of this telephone Call.
   * @exception InvalidStateException The state of some object is not valid
   * as designated by the pre-conditions for this method.
   * @exception InvalidPartyException The new party to transfer to is invalid.
   * @exception MethodNotSupportedException This method is not supported by
   * the implementation.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to invoke this method.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for the successful invocation of this method is not available.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public CallControlConnection transfer(String address)
    throws InvalidArgumentException, InvalidStateException,
      InvalidPartyException, MethodNotSupportedException,
      PrivilegeViolationException, ResourceUnavailableException,
      PlatformException;

  
  /**
   * Sets the TerminalConnection which is current acting as the "conference
   * controller." The conference controller is a TerminalConnection on a
   * telephone Call object whose Terminal is also part of another telephone
   * call. When performing a conference of two telephone Call objects, this
   * TerminalConnection and the Terminal's TerminalConnection which is part
   * of the second call are merged into one in the merged Call object.
   * <p>
   * The Call must be in the ACTIVE state in order for this method not to
   * throw an exception. The TerminalConnection supplied as the conference
   * controller must be in the Call and must not be in the DROPPED state. The
   * Provider must also be IN_SERVICE. These pre-conditions are given below.
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>termconn union (call.getConnections()).getTerminalConnections() ==
   * termconn
   * <LI>termconn.getState() != DROPPED
   * </OL>
   * <p>
   * If these pre-conditions are met, this method sets the given
   * TerminalConnection as the conference controller, where the following
   * post-conditions are met:
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>termconn union (call.getConnections()).getTerminalConnections() ==
   * termconn
   * <LI>termconn.getState() != DROPPED
   * <LI>termconn == call.getConferenceController()
   * </OL>
   * <p>
   * @param termconn The TerminalConnection to use as the conference controller
   * @exception InvalidArgumentException The TerminalConnection object
   * provided is not associated with this Call object.
   * @exception InvalidStateException The state of some object is not valid
   * as designated by the pre-conditions for this method.
   * @exception MethodNotSupportedException This method is not supported by
   * the implementation.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for the successful invocation of this method is not available.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public void setConferenceController(TerminalConnection termconn)
    throws InvalidArgumentException, InvalidStateException,
      MethodNotSupportedException, ResourceUnavailableException,
      PlatformException;

  
  /**
   * Gets the TerminalConnection which is current acting as the "conference
   * controller." The conference controller is a TerminalConnection on a
   * telephone Call object whose Terminal is also part of another telephone
   * call. When performing a conference of two telephone Call objects, this
   * TerminalConnection and the Terminal's TerminalConnection which is part
   * of the second call are merged into one in the merged Call object.
   * <p>
   * When a Call is initially created, the conference controller is set
   * to null. This method returns non-null only if the application has
   * previously successfully invoked the setConferenceController() method.
   * If a TerminalConnection, which is currently the conference controller, 
   * moves into the DROPPED state in the Call, the conference controller is
   * then set to null once again.
   * <p>
   * This method requires that the Provider is IN_SERVICE and the call is
   * either IDLE or ACTIVE. The pre-conditions are listed below.
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == IDLE or call.getState() == ACTIVE
   * </OL>
   * <p>
   * The post conditions for this method ensure that the returned
   * TerminalConnection, if not null, is in the call and not in the DROPPED
   * state as given below, where 'termconn' is the TerminalConnection returned
   * from this method:
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == IDLE or call.getState() == ACTIVE
   * <LI>if termconn != null, then termconn is an element of
   * (call.getConnections()).getTerminalConnections()
   * <LI>if termconn != null, then termconn.getState() != DROPPED
   * </OL>
   * <p>
   * @return The current TerminalConnection acting as the conference
   * controller, null if one is not set.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public CallControlTerminalConnection getConferenceController()
    throws PlatformException;


  
  /**
   * Sets the TerminalConnection which is current acting as the "transfer
   * controller." The transfer controller is a TerminalConnection on a
   * telephone Call object whose Terminal is also part of another telephone
   * call. When performing a transfer of telephone Call objects, this
   * TerminalConnection and the Terminal's TerminalConnection which is part
   * of the second call are dropped.
   * <p>
   * The Call must be in the ACTIVE state in order for this method not to
   * throw an exception. The TerminalConnection supplied as the terminal
   * controller must be in the Call and must not be in the DROPPED state. The
   * Provider must also be IN_SERVICE. These pre-conditions are given below.
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>termconn union (call.getConnections()).getTerminalConnections() ==
   * termconn
   * <LI>termconn.getState() != DROPPED
   * </OL>
   * <p>
   * If these pre-conditions are met, this method sets the given
   * TerminalConnection as the transfer controller, where the following
   * post-conditions are met:
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>termconn union (call.getConnections()).getTerminalConnections() ==
   * termconn
   * <LI>termconn.getState() != DROPPED
   * <LI>termconn == call.getTransferController()
   * </OL>
   * <p>
   * @param termconn The TerminalConnection to use as the transfer controller
   * @exception InvalidArgumentException The TerminalConnection object
   * provided is not associated with this Call object.
   * @exception InvalidStateException The state of some object is not valid
   * as designated by the pre-conditions for this method.
   * @exception MethodNotSupportedException This method is not supported by
   * the implementation.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for the successful invocation of this method is not available.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public void setTransferController(TerminalConnection termconn)
    throws InvalidArgumentException, InvalidStateException,
      MethodNotSupportedException, ResourceUnavailableException,
      PlatformException;

  
  /**
   * Gets the TerminalConnection which is current acting as the "transfer
   * controller." The transfer controller is a TerminalConnection on a
   * telephone Call object whose Terminal is also part of another telephone
   * call. When performing a transfer of two telephone Call objects, this
   * TerminalConnection and the Terminal's TerminalConnection which is part
   * of the second call are dropped in the merged call.
   * <p>
   * When a Call is initially created, the transfer controller is set
   * to null. This method returns non-null only if the application has
   * previously successfully invoked the setTransferController() method.
   * If a TerminalConnection, which is currently the transfer controller, 
   * moves into the DROPPED state in the Call, the transfer controller is
   * then set to null once again.
   * <p>
   * This method requires that the Provider is IN_SERVICE and the call is
   * either IDLE or ACTIVE. The pre-conditions are listed below.
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == IDLE or call.getState() == ACTIVE
   * </OL>
   * <p>
   * The post conditions for this method ensure that the returned
   * TerminalConnection, if not null, is in the call and not in the DROPPED
   * state as given below, where 'termconn' is the TerminalConnection returned
   * from this method:
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == IDLE or call.getState() == ACTIVE
   * <LI>if termconn != null, then termconn is an element of
   * (call.getConnections()).getTerminalConnections()
   * <LI>if termconn != null, then termconn.getState() != DROPPED
   * </OL>
   * <p>
   * @return The current TerminalConnection acting as the transfer
   * controller, null if one is not set.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public CallControlTerminalConnection getTransferController()
    throws PlatformException;


  /**
   * Controls whether this Call object is permitted or able to perform
   * the conferencing feature. If the 'enable' argument is true, this
   * method attempts to turn on the conferencing ability. If 'false' this
   * method turns disallows the conferencing feature. If the implementation
   * is unable to support conferencing, a setConferenceEnable(true) will
   * throw an exception.
   * <p>
   * This method must be called when the Call object is first create and
   * in the IDLE state. The provider must also be in service. The pre-
   * conditions are as follows:
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == IDLE
   * </OL>
   * <p>
   * The post-conditions are as follows, given an exception is not thrown
   * and the variable 'enable' represents the value of the parameter passed
   * to this method.
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == IDLE
   * <LI>enable = call.getConferenceEnable();
   * </OL>
   * <p>
   * @param enable True turns conferencing on, false turns conferencing off.
   * @exception InvalidArgumentException The Connection object provided is
   * not associated with this Address object.
   * @exception InvalidStateException The state of some object is not valid
   * as designated by the pre-conditions for this method.
   * @exception MethodNotSupportedException This method is not supported by
   * the implementation.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to invoke this method.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public void setConferenceEnable(boolean enable)
    throws InvalidArgumentException, InvalidStateException,
      MethodNotSupportedException, PrivilegeViolationException,
      PlatformException;

  
  /**
   * Return true if conferencing is enabled, false otherwise. Applications
   * may use this method initially to obtain the default value set by the
   * implementation and may attempt to change this value using the
   * CallControlCall.setConferenceEnable() method.
   * <p>
   * @return True if conferencing is enabled, false otherwise.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public boolean getConferenceEnable()
    throws PlatformException;


  /**
   * Controls whether this Call object is permitted or able to perform
   * the transfering feature. If the 'enable' argument is true, this
   * method attempts to turn on the transfering ability. If 'false' this
   * method turns disallows the transfering feature. If the implementation
   * is unable to support transfering, a setTransferEnable(true) will
   * throw an exception.
   * <p>
   * This method must be called when the Call object is first create and
   * in the IDLE state. The provider must also be in service. The pre-
   * conditions are as follows:
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == IDLE
   * </OL>
   * <p>
   * The post-conditions are as follows, given an exception is not thrown
   * and the variable 'enable' represents the value of the parameter passed
   * to this method.
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == IDLE
   * <LI>enable = call.getTransferEnable();
   * </OL>
   * <p>
   * @param enable True turns transfering on, false turns transfering off.
   * @exception InvalidArgumentException The Connection object provided is
   * not associated with this Address object.
   * @exception InvalidStateException The state of some object is not valid
   * as designated by the pre-conditions for this method.
   * @exception MethodNotSupportedException This method is not supported by
   * the implementation.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to invoke this method.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public void setTransferEnable(boolean enable)
    throws InvalidArgumentException, InvalidStateException,
      MethodNotSupportedException, PrivilegeViolationException,
      PlatformException;

  
  /**
   * Return true if transfering is enabled, false otherwise. Applications
   * may use this method initially to obtain the default value set by the
   * implementation and may attempt to change this value using the
   * CallControlCall.setTransferEnable() method.
   * <p>
   * @return True if transfering is enabled, false otherwise.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public boolean getTransferEnable()
    throws PlatformException;


  /**
   * Begins a consultation Call with the existing Call. A consultation
   * Call is a new telephone Call placed to a telephony number address
   * and is associated with the current telephony Call. The purpose of
   * creating a consultation call is to either simply consult with another
   * party or eventually to either conference in or transfer to with the
   * existing party. The creation of the new Call behaves in a similar
   * manner as Call.connect(), however the consult() method establishes
   * a special association between the new Call and the current telephone
   * Call. The consult() method
   * exists because it is often a switch feature and establishes the
   * special association between the current telephone Call. The telephone
   * number address provided must be valid and complete.
   * <p>
   * As mentioned above, the purpose of creating a consulation Call is
   * often to perform a transfer() or conference() on these two Calls.
   * Applications must specify their purpose by first telling the switch
   * whether they intend to perform a conference() or transfer() or both
   * using the setConferenceEnable() and setTransferEnable() methods on
   * this object.
   * <p>
   * The originator of this new consultation call is given by a
   * TerminalConnection which must be part of the current Call object and
   * must be in the active state. The destination address for this new
   * call is given by the second argument. Therefore, the new call placed is
   * similar to the Call.connect() method in that the originating Terminal
   * is termconn.getTerminal() and the origination Address is
   * (termconn.getConnection()).getAddress().
   * <p>
   * This method places termconn in the HELD state and create a new Call
   * object. This Call object has two connection, one each for the
   * originating side and the destination side. Both of these Connections
   * are in the IDLE state. This new Call object is returned by the consult()
   * method. The Provider must also be IN_SERVICE. The pre-conditions of this
   * method are given below:
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>termconn is element of call.getConnections().getTerminalConnections()
   * <LI>termconn.getState() == ACTIVE
   * </OL>
   * <p>
   * The post-conditions for this method are as follows, where the Call object
   * returned is denoted by 'newcall'.
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>termconn.getState() == HELD
   * <LI>newcall.getState() == ACTIVE
   * <LI>Let c = newcall.getConnections() such that c.length == 2
   * <LI>c[0].getState == IDLE
   * <LI>c[1].getState == IDLE
   * </OL>
   * <p>
   * An application can expect that this new Call object will exhibit the
   * same Connection and TerminalConnection state transition scenarios as
   * Calls placed using Call.connect() and as described in the documentation
   * for that method.
   * <p>
   * @param termconn The controlling TerminalConnection for the consultation
   * call.
   * @param address The destination telephone number address.
   * @return The new consultation Call object.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for the successful invocation of this method is unavailable.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to place a consultation telephone call.
   * @exception InvalidArgumentException An argument provided is not valid
   * either by not providing enough information for consult() or is
   * inconsistent with another argument.
   * @exception InvalidStateException Some object required by this method is
   * not is a valid state as designated by the pre-conditions for this method.
   * @exception MethodNotSupportedException The implementation does not
   * support this method.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public CallControlCall consult(TerminalConnection termconn, String address)
    throws InvalidStateException, InvalidArgumentException,
      MethodNotSupportedException, ResourceUnavailableException,
      PrivilegeViolationException, PlatformException;


  /**
   * This overloaded version of consult() is similar to the other version
   * of consult, except it does not take a destination string address as
   * an argument. Instead, it creates a Call object with one Connection in
   * the CallControlConnection.INITIATED state. Applications may use the
   * <EM>CallControlConnection.addToAddress()</EM> method to dial the
   * destination digits.
   * <p>
   * This method places termconn in the HELD state and create a new Call
   * object. This new Call object is returned by the consult() method. The
   * Provider must also be IN_SERVICE. The pre-conditions of this
   * method are given below:
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>termconn is element of call.getConnections().getTerminalConnections()
   * <LI>termconn.getState() == ACTIVE
   * </OL>
   * <p>
   * The post-conditions for this method are as follows, where the Call object
   * returned is denoted by 'newcall'.
   * <p>
   * <OL>
   * <LI>(call.getProvider()).getState() == IN_SERVICE
   * <LI>call.getState() == ACTIVE
   * <LI>termconn.getState() == HELD
   * <LI>newcall.getState() == ACTIVE
   * <LI>Let c = newcall.getConnections() such that c.length == 1
   * <LI>c[0].getState == INITIATED
   * </OL>
   * <p>
   * @param termconn The controlling TerminalConnection for the consultation
   * call.
   * @return The new consultation Call object.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for the successful invocation of this method is unavailable.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to place a consultation telephone call.
   * @exception InvalidArgumentException An argument provided is not valid
   * either by not providing enough information for consult() or is
   * inconsistent with another argument.
   * @exception InvalidStateException Some object required by this method is
   * not is a valid state as designated by the pre-conditions for this method.
   * @exception MethodNotSupportedException The implementation does not
   * support this method.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public CallControlCall consult(TerminalConnection termconn)
    throws InvalidStateException, InvalidArgumentException,
      MethodNotSupportedException, ResourceUnavailableException,
      PrivilegeViolationException, PlatformException;
}
