/*
#pragma ident "@(#)CallControlConnection.java	1.19      97/02/06     SMI"

 * Copyright (c) 1996 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

package java.telephony.callcontrol;
import  java.telephony.*;

/**
 * The CallControlConnection interface extends the core Connection interface.
 * The CallControlConnection interface provides greater details about the
 * state of the Connection object. For many of the core Connection object
 * states, the CallControlConnection interfaces often provides several sub-
 * states of the core states. These sub-states provide more information to
 * application about what is going on. The method getCallControlState() returns
 * this more detailed state. Typically, applications with either monitor
 * the core Connection states or the CallControlConnection states.
 * <p>
 * The Connection states presented by this interface are: IDLE, OFFERING,
 * QUEUED, NETWORK_REACHED, NETWORK_ALERTING, ALERTING, INITIATED, DIALING, 
 * ESTABLISHED, DISCONNECTED, FAILED, BUSY, UNKNOWN.
 * <p>
 * The real-world definitions for these states follow. Note that these
 * definitions are not technical nor absolute, however, they give an idea
 * to the reader about what these states means:
 * <p>
 * The IDLE state is defined similiarly here as it is in the core. The IDLE
 * state is the initial and transitory state for new Connection objects.
 * <p>
 * The OFFERING state is particular to ISDN and indicates that a Call is
 * being offered. Typically, applications must accept or reject this offered
 * call.
 * <p>
 * The QUEUED state indicates that a Connection is queued at a particular
 * address. This may result when someone enables the queueing mechanism on
 * an address and the address is busy when a telephone call comes in.
 * <p>
 * The NETWORK_REACHED state indicates that the outgoing telephone call has
 * reached the network, however, further events may not be received. It is
 * up to the application whether to treat this as a connected call.
 * <p>
 * The NETWORK_ALERTING state indicates that the outgoing telephone call has
 * previously reached the network, however, further information obtained from
 * the switch indicates that this call is now alerting at some destination
 * endpoint.
 * <p>
 * The ALERTING state has the same definition as in the core.
 * <p>
 * The INITIATED state indicates that an originating Connection's telephone
 * is off-hook, however, dialing has not yet begun.
 * <p>
 * The DIALING state applies to originating Connections and indicates that
 * the process of dialing has begun but has not yet completed.
 * <p>
 * The ESTABLISHED state has a similar definition as CONNECTED does in the core.
 * It indicates that a Connection has reached its final active state in the
 * call.
 * <p>
 * The DISCONNECTED state has the same definition as in the core.
 * <p>
 * The FAILED state has the same definition as in the core.
 * <p>
 * The UNKNOWN state has the same definition as in the core.
 * <p>
 * These CallControlConnection states are defined in terms of the core
 * Connection states. In other words, if the core Connection is in a particular
 * state, then the CallControlConnection must be in one of possible several
 * states. This relationship is defined below, where the core Connection states
 * are on the left, and the CallControlConnection states are on the right.
 * <p>
 * <UL>
 * <LI>IDLE ---> IDLE
 * <LI>INPROGRESS ---> QUEUED, OFFERING
 * <LI>ALERTING ---> ALERTING
 * <LI>CONNECTED ---> INITIATED, DIALING, NETWORK_REACHED, NETWORK_ALERTING, ESTABLISHED 
 * <LI>DISCONNECTED ---> DISCONNECTED
 * <LI>FAILED ---> FAILED
 * <LI>UNKNOWN ---> UNKNOWN
 * </UL>
 */

public interface CallControlConnection extends Connection {


  /**
   * The IDLE state variable.
   */
  public static final int IDLE = 0x50;


  /**
   * The OFFERING state variable.
   */
  public static final int OFFERING = 0x51;


  /**
   * The QUEUED state variable.
   */
  public static final int QUEUED = 0x52;


  /**
   * The ALERTING state variable.
   */
  public static final int ALERTING = 0x53;


  /**
   * The INITIATED state variable.
   */
  public static final int INITIATED = 0x54;


  /**
   * The DIALING state variable.
   */
  public static final int DIALING = 0x55;


  /**
   * The NETWORK_REACHED state variable.
   */
  public static final int NETWORK_REACHED = 0x56;


  /**
   * the NETWORK_ALERTING state variable.
   */
  public static final int NETWORK_ALERTING = 0x57;


  /**
   * The ESTABLISHED state variable.
   */
  public static final int ESTABLISHED = 0x58;


  /**
   * The DISCONNECTED state variable.
   */
  public static final int DISCONNECTED = 0x59;


  /**
   * The FAILED state variable.
   */
  public static final int FAILED = 0x5A;


  /**
   * The UNKNOWN state variable.
   */
  public static final int UNKNOWN = 0x5B;


  /**
   * Returns the current CallControl state of the Connection. The return value
   * will be one of states defined above.
   * <p>
   * @return The current CallControl state of the Connection.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public int getCallControlState()
    throws PlatformException;


  /**
   * Accepts a telephone call. This method is associated most commonly
   * with ISDN-type protocols which support the OFFERING state. Incoming
   * telephony calls are sometimes first "offered" to an address for
   * acceptance. Only after a call is accepted does the standard notions
   * of alerting take place. This method returns successfully when the
   * call has been accepted and this Connection has moved into the ALERTING
   * state, otherwise an exception is thrown to signal an error.
   * <p>
   * This Connection object must be in the OFFERING state. The Provider must
   * also be IN_SERVICE. These pre-conditions are given below:
   * <p>
   * <OL>
   * <LI>((this.getCall()).getProvider()).getState() == IN_SERVICE
   * <LI>this.getCallControlState() == OFFERING.
   * </OL>
   * <p>
   * The post-conditions are given below:
   * <OL>
   * <LI>((this.getCall()).getProvider()).getState() == IN_SERVICE
   * <LI>this.getCallControlState() == ALERTING.
   * </OL>
   * <p>
   * @exception InvalidStateException The state of some object is not valid
   * as designated by the pre-conditions for this method.
   * @exception MethodNotSupportedException This method is not supported by
   * the implementation.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to invoke this method.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for the successful invocation of this method is not available.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public void accept()
    throws InvalidStateException,
      MethodNotSupportedException, PrivilegeViolationException,
      ResourceUnavailableException, PlatformException;


  /**
   * Rejects a telephone call. This method is associated most commonly
   * with ISDN-type protocols which support the OFFERING state. Incoming
   * telephony calls are sometimes first "offered" to an address for
   * acceptance. Only after a call is accepted does the standard notions
   * of alerting take place. A call may also be rejected, where the call
   * does not proceed to the alerting stage. This method returns when
   * this Connection successfully moves into the DISCONNECTED state, otherwise,
   * an exception is thrown to signal an error.
   * <p>
   * This Connection object must be in OFFERING state. The Provider must
   * also be IN_SERVICE. These pre-conditions are given below:
   * <p>
   * <OL>
   * <LI>((this.getCall()).getProvider()).getState() == IN_SERVICE
   * <LI>this.getCallControlState() == OFFERING.
   * </OL>
   * <p>
   * The post-conditions are given below:
   * <OL>
   * <LI>((this.getCall()).getProvider()).getState() == IN_SERVICE
   * <LI>this.getCallControlState() == DISCONNECTED.
   * </OL>
   * <p>
   * @exception InvalidStateException The state of some object is not valid
   * as designated by the pre-conditions for this method.
   * @exception MethodNotSupportedException This method is not supported by
   * the implementation.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to invoke this method.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for the successful invocation of this method is not available.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public void reject()
    throws InvalidStateException,
      MethodNotSupportedException, PrivilegeViolationException,
      ResourceUnavailableException, PlatformException;

 
  /**
   * Redirects a telephone call to another telephone number. This feature
   * is very similar to a transfer feature, however, the application may
   * use the redirect() method when the Connection is either in the ALERTING
   * or OFFERING CallControlConnection states. The telephone address provided
   * must be valid and complete.
   * <p>
   * The Connection must be in the OFFERING or ALERTING state.
   * The first argument, destinationAddress, must be a valid telephone number
   * to dial. The Connection must also be associated with this Address
   * object and the Provider must be IN_SERVICE. These pre-conditions are given
   * below:
   * <p>
   * <OL>
   * <LI>((this.getCall()).getProvider()).getState() == IN_SERVICE
   * <LI>this.getCallControlState() == OFFERING or ALERTING
   * <LI>destinationAddress must be a valid telephone number.
   * </OL>
   * <p>
   * If successful, this method creates a new Connection associated with the
   * Address object, as given by the destinationAddress string. This new
   * Connection is in the IDLE state and is returned by this method. The
   * Connection given as an argument moves to the DISCONNECTED state when
   * this method returns. All of its TerminalConnections move to the DROPPED
   * state as well. These post-conditions are given as follows, where
   * 'connection' is the argument, and 'newconnection' is the return value.
   * <p>
   * <OL>
   * <LI>((this.getCall()).getProvider()).getState() == IN_SERVICE
   * <LI>this.getCallControlState() == DISCONNECTED.
   * <LI>newconnection.getCallControlState() == IDLE
   * <LI>Let TerminalConnection tc[] = this.getTerminalConnections()
   * <LI>For all i, tc[i].getCallControlState() == DROPPED
   * </OL>
   * <p>
   * The newconnection object acts similar to a destination connection when
   * a telephone call is first placed. The Call.connect() method provides the
   * scenarios for this destination Connection and the possible state changes
   * it may undergo.
   * <p>
   * @param desintationAddress The Connection is rerouted to this address
   * @return The new Connection associated with the Address object of the
   * new address.
   * @exception InvalidStateException The state of some object is not valid
   * as designated by the pre-conditions for this method.
   * @exception InvalidPartyException The party to which this call is
   * redirected is not valid.
   * @exception MethodNotSupportedException This method is not supported by
   * the implementation.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to invoke this method.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for the successful invocation of this method is not available.
   * @exception PlatformException A platform-specific exception occurred.
   * @exception Exception An implementation-specific exception occurred.
   */
  public CallControlConnection redirect(String destinationAddress)
    throws InvalidStateException,
      InvalidPartyException, MethodNotSupportedException,
      PrivilegeViolationException, ResourceUnavailableException,
      PlatformException;


  /**
   * Appends additional address information onto an existing Connection.
   * This method is used when part of a telephone number has been dialed
   * and the application may complete the address with this method.
   * <p>
   * This Connection must be in the DIALING or INITIATED state and the 
   * Provider must also
   * be IN_SERVICE. These pre-conditions are given below:
   * <p>
   * <OL>
   * <LI>((this.getCall()).getProvider()).getState() == IN_SERVICE
   * <LI>this.getCallControlState() == DIALING | INITIATED.
   * </OL>
   * <p>
   * The post-conditions of this method depend upon whether the provided
   * additional addressing information is enough to complete the address.
   * This is determined by the switch. If the additional addressing information
   * is enough, the Connection moves from the DIALING state into the
   * ESTABLISHED state. If not, the Connection remains in the DIALING state.
   * The following post-conditions reflect this:
   * <p>
   * <OL>
   * <LI>((this.getCall()).getProvider()).getState() == IN_SERVICE
   * <LI>this.getCallControlState() == DIALING if the addressing information was
   * not enough as determined by the switch.
   * <LI>this.getCallControlState() == ESTABLISHED if the addressing information was
   * sufficient to complete the address, determined by the switch
   * </OL>
   * <p>
   * @param additionalAddress The additional addressing information.
   * @exception InvalidStateException The state of some object is not valid
   * as designated by the pre-conditions for this method.
   * @exception MethodNotSupportedException This method is not supported by
   * the implementation.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to invoke this method.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for the successful invocation of this method is not available.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public void addToAddress(String additionalAddress)
    throws InvalidStateException,
      MethodNotSupportedException, PrivilegeViolationException,
      ResourceUnavailableException, PlatformException;


  /**
   * Parks an active Connection to another Address. "Parking" a Connection
   * transfers that Connection to another address and drops it from the current
   * telephone call. The Call Parking feature is very much like transfering
   * a telephone call except the new address does not alert. The new Connection
   * is in a special QUEUED state. Applications may then use the pickup()
   * method to "unpark" this call. The telephone address provided must be
   * valid and complete.
   * <p>
   * This Connection must be in the ESTABLISHED state.
   * The destAddress parameter must be a complete, valid telephone number
   * which can be dialed. The Provider must also be IN_SERVICE. These pre-
   * conditions are given below:
   * <p>
   * <OL>
   * <LI>((this.getCall()).getProvider()).getState() == IN_SERVICE
   * <LI>this.getCallControlState() == ESTABLISHED
   * <LI>destAddress must be a valid and complete telephone number.
   * </OL>
   * <p>
   * This method moves the connection into the
   * DISCONNECTED state and creates a new connection attached to an Address
   * object which corresponds to the destAddress argument. This new connection
   * is in the CallControlConnection QUEUED state and is returned by this
   * method. ALl of the TerminalConnections which are part of this Connection
   * move to the DROPPED state.
   * The post conditions for this method are given below, where
   * 'newconnection' is the Connection returned by this method.
   * <p>
   * <OL>
   * <LI>((this.getCall()).getProvider()).getState() == IN_SERVICE
   * <LI>newconnection.getCallControlState() == QUEUED
   * <LI>this.getCallControlState() == DISCONNECTED
   * <LI>For all termconn = this.getTerminalConnections(),
   * termconn.getState() == DROPPED
   * </OL>
   * <p>
   * @param destAddress The address at which this connection is to be parked.
   * @return The new Connection.
   * @exception InvalidStateException The state of some object is not valid
   * as designated by the pre-conditions for this method.
   * @exception MethodNotSupportedException This method is not supported by
   * the implementation.
   * @exception InvalidPartyException The party to which to party the
   * Connection is invalid.
   * @exception PrivilegeViolationException The application does not have
   * the proper authority to invoke this method.
   * @exception ResourceUnavailableException An internal resource neccessary
   * for the successful invocation of this method is not available.
   * @exception PlatformException A platform-specific exception occurred.
   */
  public CallControlConnection park(String destAddress)
    throws InvalidStateException,
      MethodNotSupportedException, PrivilegeViolationException,
      InvalidPartyException, ResourceUnavailableException, PlatformException;
}
