/*
 * @(#)ClassRenamer.java	1.4 97/08/27
 * 
 * Copyright (c) 1995, 1996 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 * 
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
 * 
 * CopyrightVersion 1.1_beta
 * 
 */

import java.io.*;
import java.util.*;

/**
 * A brute-force utility for converting code affected by API changes.
 * There are two passes made on a file:  renaming and moving.
 * <p>
 * Renaming involves changing the name of a class (or any other 
 * valid Java id) from one name to another.  Because comparison is done
 * at a lexical level (instead of parsing the Java file), there is no
 * capability to rename only one class's method -- any identifier of the
 * same name will also be converted.  For this reason and because many
 * identifiers are commonly used words, comments are not converted.
 * <p>
 * Moving is used when a class is moved from one package to another.
 * The source file needs to have any explicit package references changed,
 * plus it needs to be scanned to see if a new import statement is needed.
 * <p>
 * Changes to be made are read from a file with the following format:
 * <p><pre><code>
 * # Comment line
 * Rename:                      # the beginning of rename statements
 * old_name=new_name            # i.e. JPanel=JBufferedPane
 * 
 * Move:                        # the beginning of move statements
 * full_class_name=new_package  # i.e. foo.bar.MyClass=foo.newbar
 * </code></pre><p>
 * If a class is both being renamed and moved, define two statements:
 * a rename statement, and a move statement <i>using the new name</i>.
 * For example, to rename foo.Test to foo.bar.MyTest, define a
 * rename statement of &qt;Test=MyTest&qt; and a move statement of
 * &qt;foo.MyTest=foo.bar&qt;.
 * <p>
 * NOTE:  we are quite aware this tool is a hack, and that more elegant
 * solutions are possible (such as using javac classes to parse the 
 * Java source and provide scoping for identifiers).  You are welcome
 * to submit a more sophisticated tool to JavaSoft, but we don't have
 * time for theorical discussions regarding the approachs used here.
 * That said, we hope this tool makes your use of Swing a little easier,
 * and we appreciate your early evaluation of our product.
 *
 * @version 1.4 08/27/97
 * @author  Thomas Ball
 */
public class ClassRenamer {

    String oldText;               // the original text
    StringBuffer newText;         // the modified text
    StringTokenizer scanner;      // the basic tokenizer
    int changes;		  // the number of changes made

    // This isn't the complete list of Java tokens, but just the ones 
    // needed to separate out identifiers.
    static final String tokens = " \t\r\n\"'![](){}.,;";

    // This Properties object is used like an associative array, where the
    // key is the tag to be renamed, while its value is the new name.
    static Properties renameList = new Properties();

    // Another Properties object used for moving classes between packages.
    static Properties moveList = new Properties();

    protected static void usage() {
	System.err.println("java ClassRenamer [-f substitution_file] " +
			   "<file1> [file2 ...]");
	System.exit(1);
    }

    protected static void error(String msg) {
	System.err.println(msg);
	System.exit(1);
    }

    public ClassRenamer() {
	newText = new StringBuffer();
    }

    public void load(File file) {
	try {
	    char[] chars = new char[(int)file.length()];
	    BufferedReader in = new BufferedReader(new FileReader(file));
	    in.read(chars);
	    in.close();
	    oldText = new String(chars);
	} catch (Exception e) {
	    error("failed reading " + file.getName() + ": " + e);
	}
    }

    protected void skipPast(String end) {
	while (scanner.hasMoreTokens()) {
	    String tok = scanner.nextToken();
	    newText.append(tok);
	    if (tok.endsWith(end)) {
		return;
	    }
	}
    }

    /**
     * For each Java identifier which has an entry in the list of ids to
     * be renamed, change it to the new name.  Comments are skipped, to
     * avoid substituting something which is also used as a general
     * description (such as "close" or "read").
     * @return the number of changes made
     */
    public int rename() {
        scanner = new StringTokenizer(oldText, tokens, true);
	newText.setLength(0);           // reset buffer
	changes = 0;
	while (scanner.hasMoreTokens()) {
	    String tok = scanner.nextToken();

	    // Java all identifiers begin with a letter.  Non-identifiers
	    // would fall through, but it's faster to check first.
	    if (Character.isLetter(tok.charAt(0))) {
		String replace = renameList.getProperty(tok);
		if (replace != null) {
		    newText.append(replace);
		    changes++;
		    continue;
		}
	    }

	    newText.append(tok);

	    // The skipPast method appends the comments or string constants
	    // to newText.
	    if (tok.equals("//")) {
		skipPast("\n");
	    } else if (tok.equals("/*") || tok.equals("/**")) {
		skipPast("*/");
	    } else if (tok.equals("\"")) {
		skipPast("\"");
	    } else if (tok.equals("'")) {
                skipPast("'");
            }
	}
	return changes;
    }

    /**
     * For each entry in the list of classes to be moved, check whether
     * the class is referenced by this file.  If so, insert an import
     * statement as needed.
     * @return the number of changes made
     */
    public int move() {
	int changes = 0;
	String text = newText.toString();
	Vector imports = new Vector();
	for (Enumeration e = moveList.keys(); e.hasMoreElements();) {
	    String fullName = (String)e.nextElement();
	    int fullLength = fullName.length();

	    int iName = fullName.lastIndexOf('.');
	    String name = fullName;
	    if (iName > 0 && iName < (fullLength - 1)) {
		name = fullName.substring(iName + 1);
	    }
	    int length = name.length();

	    String pkg = (String)moveList.get(fullName);
	    int pkgLength = pkg.length();

	    /* See if this class is referenced as a fully declared class.
	     * This may either be in an import statement or within code,
	     * but in either case no import statement is needed.
	     */
	    int iFullName = text.indexOf(fullName);
	    while (iFullName >= 0) {
		int newOffset = iFullName + fullLength;
		// Yes, this is slow and memory-wasteful, but this tool 
		// would have been written in Perl if it were available
		// on all systems.  Then it'd be *really* slow. :-)
		text = text.substring(0, iFullName) + pkg + "." + name + 
		    text.substring(newOffset);
		changes++;
		iFullName = text.indexOf(fullName, newOffset);
	    }

	    // See if the new package is imported.
	    String pkgImport = pkg + ".*";
	    if (text.indexOf(pkgImport) >= 0) {
		continue;
	    }

	    /* See if this class is referenced without its package.  If
	     * it is, an import statement is probably needed.
	     */
	    iName = text.indexOf(name);
	    int nReferences = 0;
	    while (iName >= 0) {
		nReferences++;
		iName = text.indexOf(name, iName + length);
	    }
	    if (nReferences > changes) {
		imports.addElement(pkgImport);
	    }
	}

	int nImports = imports.size();
	if (nImports > 0) {
	    // Create an import statement block.
	    String importText = "";
	    for (int i = 0; i < nImports; i++) {
		importText += "import " + (String)imports.elementAt(i) + ";\n";
		changes++;
	    }

	    /* Find last import statement, as a reasonable place for insertion.
	     * This search makes the bogus assumption that import statements 
	     * all start at the beginning of a line, but that's more reasonable
	     * than assuming the word "import" isn't present in a comment or
	     * string constant.
	     */
	    int iInsert = text.lastIndexOf("\nimport");
	    if (iInsert == -1) {
		iInsert = text.indexOf("\npackage");
	    }
	    if (iInsert > 0) {
		int iEOL = text.indexOf('\n', iInsert+7); // size of "\nimport"
		if (iEOL > 0) {
		    iInsert = iEOL + 1;
		}
		text = text.substring(0, iInsert) + importText + 
		    text.substring(iInsert);
	    } else {
		text = importText + text;
	    }

	}

	if (changes > 0) {
	    newText = new StringBuffer(text);   // commit any changes
	}
	return changes;
    }

    public void save(File file) {
	try {
	    char[] chars = newText.toString().toCharArray();
	    BufferedWriter out = new BufferedWriter(new FileWriter(file));
	    out.write(chars);
	    out.close();
	} catch (IOException e) {
	    error("failed writing " + file.getName() + ": " + e);
	}
    }

    public static void loadChanges(Reader reader, String filename) {
	try {
	    BufferedReader in = new BufferedReader(reader);
	    String line;
	    final int SECT_RENAME = 1;
	    final int SECT_MOVE = 2;
	    int section = SECT_RENAME;
	    int changes = 0;
	    while ((line = in.readLine()) != null) {

		// strip out comments
		int iHash = line.indexOf('#');
		if (iHash >= 0) {
		    line = line.substring(0, iHash);
		}

		// strip off leading and trailing whitespace
		line = line.trim();
		if (line.length() == 0) {    // blank line
		    continue;
		}

		if (line.equalsIgnoreCase("rename:")) {
		    section = SECT_RENAME;
		    continue;
		}
		if (line.equalsIgnoreCase("move:")) {
		    section = SECT_MOVE;
		    continue;
		}

		// Load all "old_name=new_name" arguments.	    
		int iEquals = line.indexOf('=');
		if (iEquals >= 0) {
		    String key = line.substring(0, iEquals);
		    String value = line.substring(iEquals + 1);
		    switch (section) {
		      case SECT_RENAME:
			  renameList.put(key, value);
			  break;
		      case SECT_MOVE:
			  moveList.put(key, value);
			  break;
		      default:
			  error("internal error: bad section");
		    }
		} else {
		    error("bad argument in " + filename + ": " + line);
		}
	    }
	    in.close();
	} catch (Exception e) {
	    error("failed loading " + filename + ": " + e);
	}

	if (renameList.isEmpty() && moveList.isEmpty()) {
	    error("no rename or move definitions in " + filename);
	}
    }

    public static void main(String[] args) {
	int iArg = 0;
	int nArgs = args.length;
	if (nArgs == 0) {
	    usage();
	}

        // Load substitution file
        Reader reader = null;
        String filename;
	if (args[iArg].equals("-f") && (++iArg < nArgs)) {
            filename = args[iArg];
            File f = new File(filename);
            if (!f.exists()) {
                error(filename + " not found.");
            }
            try {
                reader = new FileReader(f);
            } catch (FileNotFoundException fnf) {
                error("failed accessing " + filename + ": " + fnf);
            }
            ++iArg;
	} else {
            filename = "class.changes";
            InputStream is = 
                ClassRenamer.class.getResourceAsStream(filename);
            if (is == null) {
                usage();
            }
            reader = new InputStreamReader(is);
        }
	loadChanges(reader, filename);

	if (iArg == nArgs) {
	    error("no files specified");
	}

	for (; iArg < nArgs; iArg++) {
	    String fileName = args[iArg];
	    File file = new File(fileName);
	    if (!file.exists()) {
		System.err.println(fileName + " not found.");
		continue;
	    }

	    System.out.print(fileName + ": ");
	    System.out.flush();

	    ClassRenamer renamer = new ClassRenamer();
	    renamer.load(file);
	    int changes = renamer.rename();
	    changes += renamer.move();
	    if (changes > 0) {
		// Save results to new file.
		File newFile = new File(fileName + "-new");
		renamer.save(newFile);

		// Delete previous -old file, if it exists.
		File oldFile = new File(fileName + "-old");
		if (oldFile.exists()) {
		    oldFile.delete();
		}

		// Rename original file to filename-old.
		file.renameTo(oldFile);

		// Rename new file to filename.
		newFile.renameTo(new File(fileName));
		System.out.println(Integer.toString(changes) +
		    ((changes > 1) ? " changes" : " change"));
	    } else {
		System.out.println("unchanged");
	    }
	}
    }
}
